% dal - dual augmented Lagrangian method for sparse learaning/reconstruction
%
% Overview:
%  Solves the following optimization problem
%   xx = argmin f(x) + lambda*c(x)
%  where f is a user specified (convex, smooth) loss function and c
%  is a measure of sparsity (currently L1 or grouped L1)
%
% Syntax:
%  [ww, uu, status] = dal(prob, ww0, uu0, A, B, lambda, <opt>)
%
% Inputs:
%  prob   : structure that contains the following fields:
%   .obj      : DAL objective function
%   .floss    : structure with three fields (p: primal loss, d: dual loss, args: arguments to the loss functions)
%   .fspec    : function handle to the regularizer spectrum function 
%               (absolute values for L1, vector of norms for grouped L1, etc.)
%   .dnorm    : function handle to the conjugate of the regularizer function
%               (max(abs(x)) for L1, max(norms) for grouped L1, etc.)
%   .softth   : soft threshold function
%   .mm       : number of samples (scalar)
%   .nn       : number of unknown variables (scalar)
%   .ll       : lower constraint for the Lagrangian multipliers (mm x 1)
%   .uu       : upper constraint for the Lagrangian multipliers (mm x 1)
%   .Ac       : inequality constraint Ac*aa<=bc for the LMs (pp x mm)
%   .bc       :                                             (pp x  1)
%   .info     : auxiliary variables for the objective function
%   .stopcond : function handle for the stopping condition
%   .hessMult : function handle to the Hessian product function (H*x)
%   .softth : function handle to the "soft threshold" function
%  ww0    : initial solution ((nn x 1) or (ns x nc) with ns*nc=nn)
%  uu0    : initial unregularized component (nu x 1)
%  A      : design matrix (mm x nn)
%  B      : design matrix for the unregularized component (mm x nu)
%  lambda : regularization constant (scalar)
%  <opt>  : list of 'fieldname1', value1, 'filedname2', value2, ...
%   aa        : initial Lagrangian multiplier [mm,1] (default zero(mm,1))
%   tol       : tolerance (default 1e-3)
%   maxiter   : maximum number of outer iterations (default 100)
%   eta       : initial barrier parameter (default 1)
%   eps       : initial internal tolerance parameter (default 1e-4)
%   eta_multp : multiplying factor for eta (default 2)
%   eps_multp : multiplying factor for eps (default 0.5)
%   solver    : internal solver. Can be either:
%               'nt'   : Newton method with cholesky factorization (default)
%               'ntsv' : Newton method saves memory (slightly slower)
%               'cg'   : Newton method with PCG
%               'qn'   : Quasi-Newton method
%   display   : display level (0: none, 1: only the last, 2: every
%               outer iteration, (default) 3: every inner iteration)
%   iter      : output the value of ww at each iteration 
%               (boolean, default 0)
% Outputs:
%  ww     : the final solution
%  uu     : the final unregularized component
%  status : various status values
%
% Reference:
% "Dual Augmented Lagrangian Method for Efficient Sparse Reconstruction"
% Ryota Tomioka and Masashi Sugiyama
% http://arxiv.org/abs/0904.0584
% 
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt
 
 

function [xx, uu, status]=dal(prob, ww0, uu0, A, B, lambda, varargin)

opt=propertylist2struct(varargin{:});
opt=set_defaults(opt, 'aa', [],...
                      'tol', 1e-3, ...
                      'iter', 0, ...
                      'maxiter', 100,...
                      'eta', 1,...
                      'eps', 1, ...
                      'eps_multp', 0.99,...
                      'eta_multp', 2, ...
                      'solver', 'nt', ...
                      'display',2);


prob=set_defaults(prob, 'll', -inf*ones(prob.mm,1), ...
                        'uu', inf*ones(prob.mm,1), ...
                        'Ac', [], ...
                        'bc', [], ...
                        'info', [], ...
                        'finddir', []);

if opt.display>0
  if ~isempty(uu0)
    nuu = length(uu0);
    vstr=sprintf('%d+%d',prob.nn,nuu);
  else
    vstr=sprintf('%d',prob.nn);
  end
  
  lstr=func2str(prob.floss.p); lstr=lstr(6:end-1);
  fprintf(['DAL ver0.98d\n#samples=%d #variables=%s lambda=%g ' ...
             'loss=%s solver=%s\n'],prob.mm, vstr, lambda, lstr, ...
            opt.solver);
end


if opt.iter
  xx  = [[ww0(:); uu0(:)], ones(length(ww0(:))+length(uu0(:)),opt.maxiter-1)*nan];
end

res    = nan*ones(1,opt.maxiter);
fval   = nan*ones(1,opt.maxiter);
etaout = nan*ones(1,opt.maxiter);
time   = nan*ones(1,opt.maxiter);
pred   = nan*ones(1,opt.maxiter);


time0=cputime;
ww   = ww0;
uu   = uu0;
gtmp = zeros(size(ww));
if isempty(opt.aa)
  aa = zeros(prob.mm,1);
else
  aa = opt.aa;
end

eta  = opt.eta;
epsl = opt.eps;
info = prob.info;
info.solver=opt.solver;
for ii=1:opt.maxiter-1
  etaout(ii)=eta;
  time(ii)=cputime-time0;

  %% Evaluate objective and Check stopping condition
  [ret,fval(ii),spec,res(ii)]=feval(prob.stopcond, ww, uu, aa, opt.tol, prob, A, B, lambda);

  %% Display
  if opt.display>1 || opt.display>0 && ret~=0
    if ii>1
      fval1 = fval(ii-1)-pred(ii-1);
    else
      fval1 = nan;
    end
    nnz = full(sum(spec>0));
    fprintf('[[%d]] fval=%g (pred=%g) #(xx~=0)=%d res=%g\n', ii, fval(ii), ...
            fval1,...
            nnz, res(ii));
  end

  if ret~=0
    break;
  end

  %% Save the original dual variable for daltv2d
  info.aa0 = aa;
  
  %% Solve minimization with respect to aa
  % fun  = @(aa,info)prob.obj(aa, prob, ww, uu, A, AT, B, lambda, eta, info);
  args = {prob,ww,uu,A,B,lambda,eta};
  switch(opt.solver)
   case {'nt','ntsv'}
    [aa,dfval,dgg,stat] = newton(prob.obj, aa, prob.ll, prob.uu, prob.Ac, ...
                                 prob.bc, epsl, prob.finddir, info, opt.display>2, args{:});
   case 'cg'
    funh = {prob.hessMult,A,eta};
    fh = {prob.obj, funh};
    [aa,dfval,dgg,stat] = newton(fh, aa, prob.ll, prob.uu, prob.Ac, ...
                                 prob.bc, epsl, prob.finddir, info, opt.display>2, args{:});
   case 'qn'
    optlbfgs=struct('epsginfo',epsl,'display',opt.display-1);
    [aa,stat]=lbfgs(prob.obj,aa,prob.ll,prob.uu,prob.Ac,prob.bc,info,optlbfgs,args{:});
   case 'fminunc'
    optfm=optimset('LargeScale','on','GradObj','on','Hessian', ...
                   'on','TolFun',1e-16,'TolX',0,'MaxIter',1000,'display','iter');
    [aa,fvalin,exitflag]=fminunc(@(xx)objdall1fminunc(xx,prob,ww, ...
                                                      uu,A,B,lambda,eta,epsl), aa, optfm);
    stat.info=info;
    stat.ret=exitflag~=1;
   otherwise
    error('Unknown method [%s]',opt.solver);
  end
  info=stat.info;


  if isfield(prob,'Aeq')
    gtmp(:) = [A', prob.Aeq']*aa;
  else    
    gtmp(:) = A'*aa;
  end
  
  ww1     = fevals(prob.softth, ww+eta*gtmp,eta*lambda,info);

  %% Predicted decrease in the objective
  %pred(ii) = norm(ww1(:)-ww(:))^2/(2*eta);
  %if ~isempty(uu)
  %pred(ii)  = pred(ii) + 0.5*eta*norm(B'*aa)^2;
  %end
  
  %% Update primal variable
  ww      = ww1;
  if ~isempty(uu)
    if isfield(prob,'Aeq')
      uu  = uu+eta*B'*aa(1:end-prob.meq);
    else
      uu  = uu+eta*B'*aa;
    end
  end

 
  %% Update barrier parameter eta and tolerance parameter epsl
  eta     = eta*opt.eta_multp^(stat.ret==0);
  epsl    = epsl*opt.eps_multp^(stat.ret==0);
  if opt.iter
    xx(:,ii+1)=[ww(:);uu(:)];
  end
end

res(ii+1:end)=[];
fval(ii+1:end)=[];
time(ii+1:end)=[];
etaout(ii+1:end)=[];
pred(ii+1:end)=[];
if opt.iter
  xx(:,ii+1:end)=[];
else
  xx = ww;
end


status=struct('aa', aa,...
              'niter',length(res),...
              'eta', etaout,...
              'pred', pred,...
              'time', time,...
              'res', res,...
              'opt', opt, ...
              'info', info,...
              'fval', fval);
